//检查meta中的check,应对各种情况是否允许通过
import store from '@/store';
const _ = require('lodash');
const jwt = require('jsonwebtoken');

/**
 * 检查是否登录
 */
const isLogin = () => {
  // vuex中是否有用户信息
  let user = _.get(store, 'state.user');
  // 有就回去
  if (user) return true;
  // 浏览器缓存中有没有token
  const token = localStorage.getItem('token');
  // 没有也回去
  if (!token) return false;
  // 将token转换成用户信息
  user = jwt.decode(token);
  // 放到vuex中
  store.commit('setUser', user, { root: true });
  return true;
};

/**
 * 查询是否是子路由,及子路由是否需要继承父级的检查结果
 * @param {Object} to 前往的路由信息
 * @param {Array} routes 所有路由集合
 */
const checkChild = (to, routes) => {
  let result = [];
  for (const route of routes) {
    // 先判断是否是该路由 && _.get(route.path.children, 'length', 0) > 0
    if (route.path === to.path) {
      const check = getCheck(route);
      if (useCheck(check)) {
        result = false;
        break;
      } else {
        result.push(route);
        break;
      }
    } else if (_.isArray(route.children) && _.get(route.children, 'length', 0) > 0) {
      // 如果父级有需要检验的,就推进result中
      const res = checkChild(to, route.children);
      if (res) {
        // 这个返回值就是if部分:
        // 1,false:不需要检查;
        // 2,是route,需要检查;推进result
        // 继续检查是否要父级检查,该路由的parent字段,只要不是false,就带父级检查
        result.push(...res); //返回的是数组,解开都扔里
        if (to.parent !== false) {
          const pcheck = getCheck(route);
          if (pcheck) result.push(route);
        }
      }
    }
  }
  return result;
};
/**
 * 获取路由的检查字段内容
 * @param {Object} route 路由
 */
const getCheck = (route) => {
  return _.get(route, 'meta.check');
};

/**
 * 抽出检查check方法
 * @param {Any} check 路由中的check字段
 * @returns 如果不是false,那就是需要检查
 */
const useCheck = (check) => {
  return _.isBoolean(check) && !check;
};
/**
 * 错误集合解释及处理
 */
const errorList = [{ function: 'isLogin', to: 'login', message: '未检测到用户登陆' }];

export default (to, from, routes) => {
  const rs = checkChild(to, routes);
  if (_.isBoolean(rs)) return rs;
  let result = true;
  for (const route of rs) {
    const check = _.get(route, 'meta.check');
    if (useCheck(check)) continue;
    else if (_.isString(check)) {
      // 字符串为函数名,是上面某个函数的结果,执行就完事了
      result = eval(check)();
      if (!result) {
        const deal = errorList.find((f) => f.function === check);
        if (!deal) console.warn(`meta -> ${check} 情况没有对应处理的方式`);
        else {
          const { to, message } = deal;
          console.error(message);
          result = to;
        }
        break;
      }
    }
  }
  return result;
};
